@@ -1,4 +1,5 @@
-{-# LANGUAGE BangPatterns, GADTs, OverloadedStrings, RankNTypes, RecordWildCards #-}
+{-# LANGUAGE BangPatterns, CPP, GADTs, OverloadedStrings, RankNTypes,
+    RecordWildCards #-}
 -- |
 -- Module      :  Data.Attoparsec.ByteString.Internal
 -- Copyright   :  Bryan O'Sullivan 2007-2015
@@ -65,7 +66,10 @@ module Data.Attoparsec.ByteString.Internal
     , atEnd
     ) where
 
-import Control.Applicative ((<|>), (<$>))
+#if !MIN_VERSION_base(4,8,0)
+import Control.Applicative ((<$>))
+#endif
+import Control.Applicative ((<|>))
 import Control.Monad (when)
 import Data.Attoparsec.ByteString.Buffer (Buffer, buffer)
 import Data.Attoparsec.ByteString.FastSet (charClass, memberWord8)
@@ -186,11 +190,12 @@ string_ suspended f s0 = T.Parser $ \t pos more lose succ ->
   let n = B.length s
       s = f s0
   in if lengthAtLeast pos n t
-     then if s == substring pos (Pos n) t
-          then succ t (pos + Pos n) more s
-          else lose t pos more [] "string"
+     then let t' = substring pos (Pos n) t
+          in if s == f t'
+             then succ t (pos + Pos n) more t'
+             else lose t pos more [] "string"
      else let t' = Buf.unsafeDrop (fromPos pos) t
-          in if t' `B.isPrefixOf` s
+          in if f t' `B.isPrefixOf` s
              then suspended s (B.drop (B.length t') s) t pos more lose succ
              else lose t pos more [] "string"
 {-# INLINE string_ #-}
@@ -208,7 +213,9 @@ stringSuspended f s0 s t pos more lose succ =
               n  = B.length s'
           in if n >= m
              then if B.unsafeTake m s' == s
-                  then succ' t' (pos' + Pos (B.length s0)) more' s0
+                  then let o = Pos (B.length s0)
+                       in succ' t' (pos' + o) more'
+                          (substring pos' o t')
                   else lose' t' pos' more' [] "string"
              else if s' == B.unsafeTake n s
                   then stringSuspended f s0 (B.unsafeDrop n s)
